#!/usr/bin/env python
import shlex
import subprocess
import sys
import os

def passthrough(cmd):
	process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	out, err = process.communicate()
	if len(err):
		print(err.decode())
	print(out.decode())

def traceroute_directly(hostname):
	cmd = ["ping", '-c', '1', '-q', hostname]
	passthrough(cmd)
	print('------------')
	cmd = ["traceroute", hostname]
	passthrough(cmd)

def traceroute_via_ncpa(ncpa_hostname, ncpa_port, ncpa_token, data_list):
	subprocess_count = data_list.count(',') + 1
	timeout = subprocess_count * 5 + 60;
	ncpa_path = os.path.dirname(os.path.realpath(__file__)) + '/check_ncpa.py'
	cmd = [ncpa_path, '-H', ncpa_hostname, '-M', 'plugins/check_traceroute.py', '-P', ncpa_port, '-t', ncpa_token, '-T', str(timeout), '-a', data_list]
	passthrough(cmd)

def main():
	if len(sys.argv) != 2:
		raise Exception("Usage: ./check_traceroute.py 'host1[:port1:token1][,...]")

	credential_list = sys.argv[1]
	as_pair = credential_list.split(',', 1)
	if len(as_pair) == 1:
		traceroute_directly(as_pair[0])
		return

	first, rest = as_pair
	first = first.split(':', 2)
	if len(first) != 3:
		raise Exception("Missing port and token when trying to query via NCPA")

	hostname, port, token = first
	traceroute_via_ncpa(hostname, port, token, rest)

if __name__ == '__main__':
	main()