<?php 
defined('BASEPATH') OR exit('No direct script access allowed');

class Pathcli extends NA_Controller {
    
    function __construct()
    {
        parent::__construct();
        if(!$this->input->is_cli_request()) {
            die("Not a cli request.");
        }

        $this->load->model('Pathnode');
        $this->t = 'nagiosna_path_status';
        $this->load->model('Check');
    }

    function process_one($node) {
        // Get traceroute command
        $current_node = $node;
        $host_list = $current_node['hostname'];
        echo "Started processing {$current_node['name']} ($host_list)\n";

        // Read all nodes between the node we're monitoring and the NNA server
        while (intval($current_node['source']) != 1) {
            echo "Looping... current id ".$current_node['id'].", source id ".$current_node['source']."\n";
            // Yes you can do this loop in 1 query on newer MySQL/MariaDB.
            // No you cannot do it 100% correctly in 1 query on MariaDB/MySQL 5.
            // No I don't think we'll run into scaling issues by doing it this way any time soon.
            // -swolf 2023-10-23
            $current_node = $this->Pathnode->read_one($current_node['source']);
            if ($current_node !== null) {
                $host_list = $current_node['hostname'] . ':' . $current_node['port'] . ':' . $current_node['token'] . ',' . $host_list;
            }
        }

        // Sanitize inputs and run the monitoring command
        $safe_hostname = escapeshellarg($node['hostname']);
        $safe_host_list = escapeshellarg($host_list);
        $cmd = "/usr/local/nagiosna/bin/check_traceroute.py $safe_host_list";
        echo "Running command: $cmd\n";
        $output = array();
        $rc = 0;
        exec($cmd, $output, $rc);
        // We probably still want to parse output if it "failed"... we'll find out later.
        // if ($rc != 0) {
        //  return;
        // }

        // Parse output
        $insert_data = array();
        $past_ping_output = false;
        $final_reachability = 1;
        $final_rtt = 0; // TODO: use this value
        for ($i = 0; $i < count($output); $i += 1) {
            if (!$past_ping_output) {
                // Currently, check_traceroute.py writes ping output, followed by 12 hyphens, followed by traceroute output.
                // This handles all of the parsing we want to do for ping output.
                $this_line = trim($output[$i]);
                if (preg_match('/------------/', $this_line)) {
                    $past_ping_output = true;
                    $this->Pathnode->update_reachability($node['id'], $final_reachability);
                    echo "Done with ping output... (id: {$node['id']}, failures: {$final_reachability}, rtt: {$final_rtt})\n";
                    continue;
                }

                if (preg_match('/^rtt/', $this_line)) {
                    // This is gross, but the message looks like this:
                    // rtt min/avg/max/mdev = 23.620/23.620/23.620/0.000 ms
                    // and we're going to grab the second number.
                    $rtt = explode(' = ', $this_line);
                    $rtt = $rtt[1];
                    $rtt = explode('/', $rtt);
                    $rtt = $rtt[1];
                    $final_reachability = 0;
                    $final_rtt = floatval($rtt);
                    echo "Got round trip time of $final_rtt\n";
                }

                continue;
            }
            // Traceroute output parsing.

            $this_line = explode(' ', trim($output[$i]));
            if (!is_numeric($this_line[0])) {
                continue;
            }

            $hop_number = intval($this_line[0]);
            $hostname = $this_line[2];
            $ip_address = '';
            $ms = 0.0;
            if (count($this_line) > 5) {
                $ip_address = trim($this_line[3], '()');
                $ms = floatval($this_line[5]);
            }

            $reachability = 0;
            if ($hop_number == '*' || $hostname == "*" || $ip_address == "*" || $ms == "*") {
                $reachability = 1;
            }

            $insert_data[] = array(
                'hostname' => $hostname,
                'reachability' => $reachability,
                'rtt' => $ms,
                'hop_number' => $hop_number,
                'modified' => date('Y-m-d H:i:s'),
                'target_node' => $node['id'],
            );
        }

        $delete_query = "DELETE FROM " . $this->t . " WHERE target_node = " . $this->db->escape($node['id']);
        echo "Deleting status nodes via $delete_query\n";
        $result = $this->db->query($delete_query);

        $insert_query = "INSERT INTO " . $this->t . " (hostname,reachability,rtt,hop_number,modified,target_node) VALUES ";
        foreach ($insert_data as $i => $datum) {
            if ($i != 0) { 
                $insert_query .= ',';
            }
            $insert_query .= "(" . 
                $this->db->escape($datum['hostname']) . "," .
                $this->db->escape($datum['reachability']) . "," .
                $this->db->escape($datum['rtt']) . "," .
                $this->db->escape($datum['hop_number']) . "," .
                $this->db->escape($datum['modified']) . "," .
                $this->db->escape($datum['target_node']) . 
            ")";
        }

        echo "Inserting new status nodes via $insert_query\n";
        $result = $this->db->query($insert_query);

        // Alerting criteria for path nodes.

        $check_array = $this->Check->read_by_associated_path($node['id']);
        foreach ($check_array as $check) {
            $warning_threshold = 0;
            $critical_threshold = 0;
            if ($check['metric'] !== 'lost_connection') {
                $warning_threshold = floatval($check['warning']);
                $warning_threshold = floatval($check['critical']);
            }
            $last_hop = $insert_data[count($insert_data) - 1];
            // Replace with switches if we add extra check types.
            $candidate_metric = ($check['metric'] === 'lost_connection' ? $final_reachability : $last_hop['rtt']);
            $intermediate_metric_key = ($check['metric'] === 'lost_connection' ? 'reachability' : 'rtt');

            $status_code = 0; // ($candidate_metric > $critical_threshold) + ($candidate_metric > $warning_threshold) ? or is that crazypants
            if ($candidate_metric > $warning_threshold) { $status_code = 1; }
            if ($candidate_metric > $critical_threshold) { $status_code = 2; }

            echo "Alerting on {$check['metric']}\n";
            $pn_hostname = $safe_hostname;
            $pn_hop_number = count($insert_data) - 1;
            $stdout = "$safe_hostname is reachable";
            if ($check['metric'] === 'rtt') {
                $stdout = 'Round-trip time was ' . intval($final_rtt) . 'ms';
            }

            if ($status_code != 0) {
                $last_good_node = array('hop_number' => -1, 'hostname' => '(none)');
                // Read through the intermediate nodes and figure out which one was known to be okay
                for ($i = count($insert_data) - 1; $i >= 0; $i -= 1) {
                    $current_node = $insert_data[$i];
                    if (intval($current_node[$intermediate_metric_key]) < $warning_threshold && $current_node['hostname'] !== '*') {
                        $last_good_node = $current_node;
                        break;
                    }

                    $stdout = "$safe_hostname is not reachable"; // lost_connection
                    if ($check['metric'] === 'rtt') {
                        $stdout = 'Round-trip time was ' . intval($last_hop['rtt']) . 'ms';
                    }
                }
                $pn_hostname = escapeshellarg($last_good_node['hostname']);
                $pn_hop_number = intval($last_good_node['hop_number']);
            }

            $pn_cid = intval($check['cid']);
            $pn_status_code = intval($status_code);
            $pn_stdout = escapeshellarg($stdout);
            $cmd = "/usr/local/nagiosna/bin/path_notify.py $pn_cid $pn_hostname $pn_hop_number $pn_status_code $pn_stdout";
            echo "Running $cmd\n";
            exec($cmd, $output, $rc);
        }

        return;
    }
    
    function index()
    {
        $max_time = 50;
        $sleep_time = 15; // in seconds
        $this->logging = 1;
        $start_time = time();

        $this->load->model("Pathnode");
        $all_nodes = $this->Pathnode->read_all();
        foreach($all_nodes as $node) {
            if ($node['type'] == 2) {
                // TODO: type is hardcoded integer... 0/1/2 Basic/NCPA/NNA 
                echo "We are skipping monitoring of ". $node['hostname'] . "...\n";
                continue;
            }
            $this->process_one($node);
        }
    }
}
